//+------------------------------------------------------------------+
//|                                             SSMultiFilter_v1.mq4 |
//|                                                        Gary Hill |
//|                                                     Feb. 1, 2009 |
//+------------------------------------------------------------------+

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_maximum 5
#property indicator_minimum -5

//---- Buffers
double MaxMacdBuffer[];
double StochBuffer[];
double ZZiBuffer[];
double AdvancedMacdBuffer[];

//---- Filter Parameter Settings
extern string Remark1 = "== Max Macd Filter Settings ==";
extern bool ShowMacdVLines = false;  //show filter value as vertical lines
extern bool ShowMacdHLines = true;  //show filter value as horizontal lines                       
extern int TimeFrame  = 0;
extern int FastEMA = 12;
extern int SlowEMA = 26;
extern int SignalSMA = 9;
extern int applied_price = 0;
extern int maxtrendlines = 100;  //maximum number of visible vertical trendlines on the chart
//---- stochastic input parameters
extern string Remark2 = "== Stochastic Filter Settings ==";
extern bool ShowStochVLines = false;  //show filter value as vertical lines 
extern bool ShowStochHLines = true;  //show filter value as horizontal lines 
extern int KPeriod = 8;
extern int DPeriod = 3;
extern int Slowing = 3;
extern int stoch_maxtrendlines = 100;   //maximum number of visible vertical trendlines on the chart
//---- ZZi input parameters
extern string Remark3 = "== ZZi Filter Settings ==";
extern bool ShowZZiVLines = false;  //show filter value as vertical lines
extern bool ShowZZiHLines = true;  //show filter value as horizontal lines  
extern int CCIPeriod = 14;
extern int zzi_maxtrendlines = 100;   //maximum number of visible vertical trendlines on the chart
//---- Advanced Macd  input parameters
extern string Remark4 = "== Advanced Macd Filter Settings ==";
extern bool ShowAMacdVLines = false;  //show filter value as vertical lines
extern bool ShowAMacdHLines = false;  //show filter value as horizontal lines  
//extern int space reserved for Advanced Macd parameter values; need to be added - Not yet available
extern int amacd_maxtrendlines = 100;   //maximum number of visible vertical trendlines on the chart


//---- Variables
int bluemarker1, bluemarker2;
int redmarker1, redmarker2;
double bluenew, blueold;
double rednew, redold;
double macd_blue, macd_red;
string trendline;
int trend, counter, counter2;

double High1, High2;
double Low1, Low2;
double s1,s2,s3,s4,s5,s6;
string strendline;
int strend, scounter, scounter2;

double HHigh1, HHigh2;
double LLow1, LLow2;
double ss1,ss2,ss3,ss4,ss5,ss6;
string ztrendline;
int ztrend, zcounter, zcounter2;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   int Periods = 1;
   SetIndexEmptyValue(0,0.0);
   SetIndexEmptyValue(1,0.0);
   SetIndexEmptyValue(2,0.0);
   SetIndexEmptyValue(3,0.0);


   //---- INDICATOR STYLE
   SetIndexStyle( 0, DRAW_LINE, STYLE_SOLID, 1, Red);
   SetIndexDrawBegin(0, Periods);
   if(ShowMacdHLines == true) SetIndexBuffer(0, MaxMacdBuffer);
   
   SetIndexStyle( 1, DRAW_LINE, STYLE_SOLID, 1, Blue);
   SetIndexDrawBegin(1, Periods);
   SetIndexShift(1,2);
   if(ShowStochHLines == true) SetIndexBuffer(1, StochBuffer);
   
   
   SetIndexStyle( 2, DRAW_LINE, STYLE_SOLID, 1, Green);
   SetIndexDrawBegin(2, Periods);
   SetIndexShift(2,2);
   if(ShowZZiHLines == true) SetIndexBuffer(2, ZZiBuffer);
   
   SetIndexStyle( 3, DRAW_LINE, STYLE_SOLID, 1, Yellow);
   SetIndexDrawBegin(3, Periods);
   SetIndexBuffer(3, AdvancedMacdBuffer);
   if(ShowAMacdHLines == true) SetIndexBuffer(2, ZZiBuffer);
   
   SetLevelValue(1,0.0);
   SetLevelStyle(STYLE_DOT,1,Silver);
   IndicatorDigits(Digits);
   IndicatorShortName("SS Multi Filters   ");

  
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   ObjectsDeleteAll(0, OBJ_VLINE);
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   //---- Find MaxRecords
   int counted_bars=IndicatorCounted();
   if(counted_bars<0) return(-1);
   if(counted_bars>0) { counted_bars--; }
   int MaxRecords = Bars-counted_bars;

//+------------------------------------------------------------------+
//| MaxMacdFilter                                                    |
//+------------------------------------------------------------------+
   for( int a = 1; a < MaxRecords ; a++ )
   {
      bluemarker1 = 0; bluemarker2 = 0; 
      redmarker1 = 0; redmarker2 = 0;
      bluenew = 0; blueold = 0;
      rednew = 0; redold = 0;
      
      High1 = 0; High2 = 0; 
      Low1 = 0; Low2 = 0;
      s1 = 0; s2 = 0; s3 = 0; s4 = 0;
      
      HHigh1 = 0; HHigh2 = 0; 
      LLow1 = 0; LLow2 = 0;
      ss1 = 0; ss2 = 0; ss3 = 0; ss4 = 0;
      
      for (int i=1; i<=500; i++)
      {
         double macd3 = (iCustom(NULL, 0, "MACD_inColor",TimeFrame,FastEMA,SlowEMA,SignalSMA,applied_price,2,i)); 
         double macd33 = iCustom(NULL, 0, "MACD_inColor",TimeFrame,FastEMA,SlowEMA,SignalSMA,applied_price,2,i+1);
         double macd4 = iCustom(NULL, 0, "MACD_inColor",TimeFrame,FastEMA,SlowEMA,SignalSMA,applied_price,3,i);
         double macd44 = iCustom(NULL, 0, "MACD_inColor",TimeFrame,FastEMA,SlowEMA,SignalSMA,applied_price,3,i+1);
         
         if (bluemarker1 == 0)   // testing for first blue set
         {
            if (macd3 > 0 && macd33 == 0)  //ox    start of blueset1
            { 
               bluemarker1 = 1; 
               if (macd3 > bluenew) bluenew = macd3;
            }
            if (macd3 > 0 && macd33 > 0)    // xx  values within blueset1
            {
               if (macd3 >= macd33 && macd3 >= bluenew) bluenew = macd3;
               else if (macd33 > bluenew) bluenew = macd33;
            }    
            if (macd3 == 0 && macd33 > 0)       // xo   end of blueset1
            {
               if (macd33 > bluenew) bluenew = macd33;
            }
         }
   
         else if (bluemarker1 == 1)   // testing for second blue set
         {
            if (macd3 > 0 && macd33 == 0)  //ox    start of blueset2
            { 
               bluemarker2 = 1; 
               if (macd3 > blueold) blueold = macd3;
            }
            if (macd3 > 0 && macd33 > 0)    // xx  values within blueset2
            {
               if (macd3 >= macd33 && macd3 >= blueold) blueold = macd3;
               else if (macd33 > blueold) blueold = macd33;
            }    
            if (macd3 == 0 && macd33 > 0)       // xo   end of blueset2
            {
               if (macd33 > blueold) blueold = macd33;
            }
         }                                           
         
         if (redmarker1 == 0)   // testing for first red set
         {
            if (macd4 < 0 && macd44 == 0)  //ox    start of redset1
            { 
               redmarker1 = 1; 
               if (macd4 < rednew) rednew = macd4;
            }
            if (macd4 < 0 && macd44 < 0)    // xx  values within redset1
            {
               if (macd4 <= macd44 && macd4 <= rednew) rednew = macd4;
               else if (macd44 < rednew) rednew = macd44;           
            }    
            if (macd4 == 0 && macd44 < 0)       // xo   end of redset1
            {
               if (macd44 < rednew) rednew = macd44;
            }
         }
            
         else if (redmarker1 == 1)   // testing for second red set
         {
            if (macd4 < 0 && macd44 == 0)  //ox    start of redset2
            { 
               redmarker2 = 1; 
               if (macd4 < redold) redold = macd4;
            }
            if (macd4 < 0 && macd44 < 0)    // xx  values within redset2
            {
               if (macd4 <= macd44 && macd4 <= redold) redold = macd4;
               else if (macd44 < redold) redold = macd44;
            }    
            if (macd4 == 0 && macd44 < 0)       // xo   end of redset2
            {
               if (macd44 < redold) redold = macd44;
            }
         }
         
         if (bluemarker1 == 1 && bluemarker2 == 1 && redmarker1 == 1 && redmarker2 ==1) // = 2 sets each of the most recent blue and red bars have been analyzed
         {         
            double macdblue = iCustom(NULL, 0, "MACD_inColor",TimeFrame,FastEMA,SlowEMA,SignalSMA,applied_price,2,1); 
            double macdred = iCustom(NULL, 0, "MACD_inColor",TimeFrame,FastEMA,SlowEMA,SignalSMA,applied_price,3,1); 
      
            if (trend == 0 || trend == -1 && macdblue > 0 && bluenew > blueold)
            {
               trend = 1;
               MaxMacdBuffer[a-1] = trend;
               counter = counter + 1;
               counter2 = counter - maxtrendlines;
               if(counter > maxtrendlines) ObjectDelete("trendline"+counter2); 
               datetime time = iTime(NULL,0,1);
               if(ShowMacdVLines == true) ObjectCreate("trendline"+counter, OBJ_VLINE,0,time,0);
               ObjectSet("trendline"+counter,OBJPROP_COLOR,Blue); 
            }
            else if (trend == 0 || trend == 1 && macdred < 0 && rednew < redold)   
            {
               trend = -1;
               MaxMacdBuffer[a-1] = trend;
               counter = counter + 1;
               counter2 = counter - maxtrendlines;
               if(counter > maxtrendlines) ObjectDelete("trendline"+counter2);
               time = iTime(NULL,0,1);
               if(ShowMacdVLines == true) ObjectCreate("trendline"+counter, OBJ_VLINE,0,time,0);
               ObjectSet("trendline"+counter,OBJPROP_COLOR,Red);
            }          
            break;
         }     
      }
   MaxMacdBuffer[a-1] = trend;                
   }
   
//+------------------------------------------------------------------+
//| Stochastic Filter                                                |
//+------------------------------------------------------------------+
       
         for (i=1; i<=500; i++)
         {
            double value1 = iCustom(NULL, 0, "Stochastic",KPeriod,DPeriod,Slowing,0,i);
            double value2 = iCustom(NULL, 0, "Stochastic",KPeriod,DPeriod,Slowing,0,i+1);
            double value3 = iCustom(NULL, 0, "Stochastic",KPeriod,DPeriod,Slowing,0,i+2);
            
            if(s1 == 0 && s2 == 0 && s3 == 0 && s4 == 0)
            {
               if(value1 > value2 && value2 < value3) s1 = value2;
               if(value1 < value2 && value2 > value3) s1 = value2;
            }
            else if(s1 != 0 && s2 == 0 && s3 == 0 && s4 == 0)
            {
               if(value1 > value2 && value2 < value3) s2 = value2;
               if(value1 < value2 && value2 > value3) s2 = value2;
            }
            else if(s1 != 0 && s2 != 0 && s3 == 0 && s4 == 0)
            {
               if(value1 > value2 && value2 < value3) s3 = value2;
               if(value1 < value2 && value2 > value3) s3 = value2;
            }
            else if(s1 != 0 && s2 != 0 && s3 != 0 && s4 == 0)
            {
               if(value1 > value2 && value2 < value3) s4 = value2;
               if(value1 < value2 && value2 > value3) s4 = value2;
            }
            else if(s1 != 0 && s2 != 0 && s3 != 0 && s4 != 0) break;
         }
         
         if(s1 > s2)
         {
            High1 = s1;
            Low1 = s2;
            High2 = s3;
            Low2 = s4;
         }
         if(s1 < s2)
         {
            Low1 = s1;
            High1 = s2;
            Low2 = s3;
            High2 = s4;
         }
         
         double value4 = iCustom(NULL, 0, "Stochastic",KPeriod,DPeriod,Slowing,0,1);
               
         if(s1 == High1)
         {
            if(value4 < Low1) strend = -2;                    // = Breakout low = short trade     
            if(value4 < Low1 && High1 < High2) strend = -2;   // = Trade continuation = short trade
         }
         if(s1 == Low1)
         { 
            if(value4 > High1) strend = 2;                    // = Breakout high = long trade   
            if(value4 > High1 && Low1 > Low2) strend = 2;     // = Trade continuation = long trade 
         }
         StochBuffer[a] = strend;
         int PriorTrend = StochBuffer[a+1];
            
         if(strend == 2 && PriorTrend < strend)
         {
            scounter = scounter + 1;
            scounter2 = scounter - stoch_maxtrendlines;
            if(scounter > stoch_maxtrendlines) ObjectDelete("strendline"+scounter2); 
            time = iTime(NULL,0,1);
            if(ShowStochVLines == true) ObjectCreate("strendline"+scounter, OBJ_VLINE,0,time,0);
            ObjectSet("strendline"+scounter,OBJPROP_COLOR,Blue);             
         }
         else if(strend == -2 && PriorTrend > strend)
         {
            scounter = scounter + 1;
            scounter2 = scounter - stoch_maxtrendlines;
            if(scounter > stoch_maxtrendlines) ObjectDelete("trendline"+scounter2);
            time = iTime(NULL,0,1);
            if(ShowStochVLines == true) ObjectCreate("trendline"+scounter, OBJ_VLINE,0,time,0);
            ObjectSet("trendline"+scounter,OBJPROP_COLOR,Red);
         }
         
//+------------------------------------------------------------------+
//| ZZiFilter                                                        |
//+------------------------------------------------------------------+ 

   for (int ii=1; ii<=500; ii++)
         {
            double vvalue1 = iCustom(NULL, 0, "Zee Zee i",CCIPeriod,0,ii);
            double vvalue2 = iCustom(NULL, 0, "Zee Zee i",CCIPeriod,0,ii+1);
            double vvalue3 = iCustom(NULL, 0, "Zee Zee i",CCIPeriod,0,ii+2);
            
            if(ss1 == 0 && ss2 == 0 && ss3 == 0 && ss4 == 0)
            {
               if(vvalue1 > vvalue2 && vvalue2 < vvalue3) ss1 = vvalue2;
               if(vvalue1 < vvalue2 && vvalue2 > vvalue3) ss1 = vvalue2;
            }
            else if(ss1 != 0 && ss2 == 0 && ss3 == 0 && ss4 == 0)
            {
               if(vvalue1 > vvalue2 && vvalue2 < vvalue3) ss2 = vvalue2;
               if(vvalue1 < vvalue2 && vvalue2 > vvalue3) ss2 = vvalue2;
            }
            else if(ss1 != 0 && ss2 != 0 && ss3 == 0 && ss4 == 0)
            {
               if(vvalue1 > vvalue2 && vvalue2 < vvalue3) ss3 = vvalue2;
               if(vvalue1 < vvalue2 && vvalue2 > vvalue3) ss3 = vvalue2;
            }
            else if(ss1 != 0 && ss2 != 0 && ss3 != 0 && ss4 == 0)
            {
               if(vvalue1 > vvalue2 && vvalue2 < vvalue3) ss4 = vvalue2;
               if(vvalue1 < vvalue2 && vvalue2 > vvalue3) ss4 = vvalue2;
            }
            else if(ss1 != 0 && ss2 != 0 && ss3 != 0 && ss4 != 0) break;
         }
         
         if(ss1 > ss2)
         {
            HHigh1 = ss1;
            LLow1 = ss2;
            HHigh2 = ss3;
            LLow2 = ss4;
         }
         if(ss1 < ss2)
         {
            LLow1 = ss1;
            HHigh1 = ss2;
            LLow2 = ss3;
            HHigh2 = ss4;
         }
         
         double vvalue4 = iCustom(NULL, 0, "Zee Zee i",CCIPeriod,0,1);
               
         if(ss1 == HHigh1)
         {
            if(vvalue4 < LLow1) ztrend = -3;                    // = Breakout low = short trade     
            if(vvalue4 < LLow1 && HHigh1 < HHigh2) ztrend = -3;   // = Trade continuation = short trade
         }
         if(ss1 == LLow1)
         { 
            if(vvalue4 > HHigh1) ztrend = 3;                    // = Breakout high = long trade   
            if(vvalue4 > HHigh1 && LLow1 > LLow2) ztrend = 3;     // = Trade continuation = long trade 
         }
            
         ZZiBuffer[a] = ztrend;
         int PPriorTrend = ZZiBuffer[a+1];
            
         if(ztrend == 3 && PPriorTrend < ztrend)
         {
            zcounter = zcounter + 1;
            zcounter2 = zcounter - zzi_maxtrendlines;
            if(zcounter > zzi_maxtrendlines) ObjectDelete("ztrendline"+zcounter2); 
            time = iTime(NULL,0,1);
            if(ShowZZiVLines == true) ObjectCreate("ztrendline"+zcounter, OBJ_VLINE,0,time,0);
            ObjectSet("ztrendline"+zcounter,OBJPROP_COLOR,Blue);             
         }
         else if(ztrend == -3 && PPriorTrend > ztrend)
         {
            zcounter = zcounter + 1;
            zcounter2 = zcounter - zzi_maxtrendlines;
            if(zcounter > zzi_maxtrendlines) ObjectDelete("trendline"+zcounter2);
            time = iTime(NULL,0,1);
            if(ShowZZiVLines == true) ObjectCreate("ztrendline"+zcounter, OBJ_VLINE,0,time,0);
            ObjectSet("ztrendline"+zcounter,OBJPROP_COLOR,Red);
         }
         
//+------------------------------------------------------------------+
//| Advanced Macd Filter - space reserved; not yet available         |
//+------------------------------------------------------------------+          
              
   return(0);
  }
//+------------------------------------------------------------------+